using System.Collections.Generic;
using SimpleJSON;
using MacGruber;

namespace CheesyFX
{
    public class FlexIt : MVRScript
    {
        private static string morphPath = "Custom/Atom/Person/Morphs/female/CheesyFX/FlexIt/";
        private List<FlexRegion> flexRegions = new List<FlexRegion>();
        
        private static GenerateDAZMorphsControlUI morphUI;
        private UIDynamicTabBar regionTabbar;

        private string[] regionNames = { "Knee" };

        public override void Init()
        {
            DAZCharacterSelector dcs = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
            if (dcs.selectedCharacter.isMale)
            {
                morphPath = morphPath.Replace("female", "male");
            }
            if (Utils.IsInPackage(this))
            {
                var packageUid = Utils.GetPluginPath(this).Split(':')[0] + ":/";
                // factoryDefaults = packageUid + factoryDefaults;
                morphPath = packageUid + morphPath;
            }
            morphUI = dcs.morphsControlUI;
            
            for(int i=0; i<regionNames.Length; i++)
            {
                var flexRegion = gameObject.AddComponent<FlexRegion>();
                flexRegion.Init(0, regionNames[i], "Shin");
                flexRegions.Add(flexRegion);
            }
            
            UIManager.OnInitUI(CreateUIElement, leftUIElements, rightUIElements);
            CreateUI();
        }

        private void CreateUI()
        {
            regionTabbar = UIManager.CreateTabBar(regionNames, delegate(int i) {  }, script:this, invokeDefault:true);
            var flexRegion = flexRegions[0];
            CreateSlider(flexRegion.bAccelStrengthJSON);
            CreateSlider(flexRegion.bGravStrengthJSON);
            CreateSlider(flexRegion.bMinAccelJSON);
            CreateSlider(flexRegion.bMaxMorphJSON);
            
            CreateSlider(flexRegion.sAccelStrengthJSON, true);
            CreateSlider(flexRegion.sGravStrengthJSON, true);
            CreateSlider(flexRegion.sMinAccelJSON, true);
            CreateSlider(flexRegion.sMaxMorphJSON, true);
            
        }

        public void FixedUpdate()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].Flex();
            }
        }

        public void OnDisable()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].OnDisable();
            }
        }
        
        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].Reset();
            }
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, needsStore);
            return jc;
        }

        public static DAZMorph GetMorphByUid(string name)
        {
            DAZMorph morph;
            string uid = morphPath + name +".vmi";
            morph = morphUI.GetMorphByUid(uid);
            if(morph == null) $"Morph '{uid}' not found!".Print();
            return morph;
        }
    }
}