using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class FlexRegion : MonoBehaviour
    {
        private RBDynamics[] rbDynamics = new RBDynamics[2];
        private int id;
        private DAZMorph[] bMorphs = new DAZMorph[2];
        private DAZMorph[] sMorphs = new DAZMorph[2];
        public JSONStorableFloat bAccelStrengthJSON;
        public JSONStorableFloat sAccelStrengthJSON;
        public JSONStorableFloat bGravStrengthJSON;
        public JSONStorableFloat sGravStrengthJSON;
        public JSONStorableFloat bMaxMorphJSON;
        public JSONStorableFloat sMaxMorphJSON;
        public JSONStorableFloat bMinAccelJSON;
        public JSONStorableFloat sMinAccelJSON;

        private float speed = Time.fixedUnscaledDeltaTime * 10f;
        Transform world = SuperController.singleton.transform;
        private Dictionary<int, string> sides = new Dictionary<int, string>
        {
            { 0, "L" }, { 1, "R" }
        };

        private float bAccelStrength { get; set; }
        private float sAccelStrength { get; set; }
        private float bGravStrength { get; set; }
        private float sGravStrength { get; set; }
        private float bMinAccel { get; set; }
        private float sMinAccel { get; set; }

        public void Init(int id, string region, string rbName)
        {
            this.id = id;
            var atom = gameObject.GetAtom();
            Rigidbody rb;
            for (int i = 0; i < 2; i++)
            {
                string side = sides[i];
                rb = atom.forceReceivers.FirstOrDefault(x => x.name == side.ToLower() + rbName)?.GetComponent<Rigidbody>();
                rbDynamics[i] = rb.AddRbDynamic(i);
                bMorphs[i] = FlexIt.GetMorphByUid($"FI_{region}Bend{side}");
                sMorphs[i] = FlexIt.GetMorphByUid($"FI_{region}Straighten{side}");
                bMorphs[i].jsonFloat.constrained = true;
                sMorphs[i].jsonFloat.constrained = true;
            }
            
            bAccelStrengthJSON = new JSONStorableFloat("Bend acceleration strength", 5f, val => bAccelStrength = val*0.01f, 0f, 20f, false);
            sAccelStrengthJSON = new JSONStorableFloat("Straighten acceleration strength", 8f, val => sAccelStrength = val*0.01f, 0f, 20f, false);
            bGravStrengthJSON = new JSONStorableFloat("Bend gravity strength", 3f, val => bGravStrength = val*0.1f, 0f, 20f, false);
            sGravStrengthJSON = new JSONStorableFloat("Straighten gravity strength", 5f, val => sGravStrength = val*0.1f, 0f, 20f, false);
            bMinAccelJSON = new JSONStorableFloat("Bend min accel", 10f, val => bMinAccel = val, 0f, 20f, false);
            sMinAccelJSON = new JSONStorableFloat("Straighten min accel", 10f, val => sMinAccel = val, 0f, 20f, false);
            bMaxMorphJSON = new JSONStorableFloat("Bend max morph", 1.6f, val =>
            {
                bMorphs[0].jsonFloat.max = val;
                bMorphs[1].jsonFloat.max = val;
            }, 0f, 3f, false);
            sMaxMorphJSON = new JSONStorableFloat("Straighten max morph", 2f, val =>
            {
                sMorphs[0].jsonFloat.max = val;
                sMorphs[1].jsonFloat.max = val;
            }, 0f, 3f, false);
            bAccelStrength = bAccelStrengthJSON.defaultVal * .01f;
            sAccelStrength = sAccelStrengthJSON.defaultVal * .01f;
            bGravStrength = bGravStrengthJSON.defaultVal * .1f;
            sGravStrength = sGravStrengthJSON.defaultVal * .1f;
            bMinAccel = bMinAccelJSON.defaultVal;
            sMinAccel = sMinAccelJSON.defaultVal;
            // bMaxMorph = bMaxMorphJSON.defaultVal;
            // sMaxMorph = sMaxMorphJSON.defaultVal;
        }

        public void Flex()
        {
            
            for(int i=0; i<2; i++)
            {
                float m = bMorphs[i].morphValue;
                if(m > .001f) bMorphs[i].morphValue = Mathf.Lerp(m, 0f, speed);
                m = sMorphs[i].morphValue;
                if(m > .001f) sMorphs[i].morphValue = Mathf.Lerp(m, 0f, speed);
                float f = rbDynamics[i].angularAcceleration.x;
                if(f > 10f)
                {
                    f *= bAccelStrength;
                    f += bMorphs[i].morphValue;
                    bMorphs[i].morphValue = f;
                }
                else if(f < -10f)
                {
                    f = Mathf.Abs(f) * sAccelStrength;
                    f += sMorphs[i].morphValue;
                    sMorphs[i].morphValue = f;
                }
            }

            float bend = Vector3.Dot(rbDynamics[0].rigidbody.transform.forward, world.up);
            if(bend < 0f)
                bMorphs[0].morphValue += -bend * bGravStrength;
            else
                sMorphs[0].morphValue += bend * sGravStrength;
        }

        public void Reset()
        {
            for(int i=0; i<2; i++)
            {
                bMorphs[i].morphValue = 0f;
                sMorphs[i].morphValue = 0f;
            }
        }

        public void OnDisable()
        {
            for (int i = 0; i < 2; i++)
            {
                rbDynamics[i].uses--;
            }
            Reset();
        }
    }
}